class String

  # Returns first _n_ separations. The default
  # separator is <tt>$/</tt>. If a zero-length record
  # separator is supplied, the string is split on /\n+/.
  # If the record separator is set to <tt>nil</tt>,
  # then the string is split on /\s+/.
  #
  #   require 'facet/string/first&last'
  #
  #   "Hello World".first(3)  #=> "Hel"
  #
  def first(n=1, separator=$/)
    separator = /\n+/ if separator == ''
    separator = /\s+/ unless separator
    separator = Regexp.new( Regexp.escape( separator ) ) if String === separator
    i,s = 0,0
    n.times { i = self.index(separator,s); s = $~.end(0) }
    return self[0...i]
  end

  # Returns last _n_ separations. If a zero-length record
  # separator is supplied, the string is split on /\n+/.
  # If the record separator is set to <tt>nil</tt>, then
  # the string is split on /\s+/.
  #
  #   require 'facet/string/first&last'
  #
  #   "Hello World".last(1,nil)  #=> "World"
  #
  def last(n=1, separator=$/)
    separator = /\n+/ if separator == ''
    separator = /\s+/ unless separator
    separator = Regexp.new( Regexp.escape( separator ) ) if String === separator
    rself = self.reverse
    i,s = 0,0
    n.times { i = rself.index(separator,s); s = $~.end(0) }
    return rself[0...i].reverse
  end
  
  # Replaces the first seperation with a new string.
  # If a zero-length record separator is supplied,
  # the string is split on /\n+/. If the record 
  # separator is set to <tt>nil</tt>, then the 
  # string is split on characters.
  #
  #   require 'facet/string/first&last'
  #
  #   $/ = nil
  #   "Hello World".first = "Hello,"  #=> "Hello, Hello World"
  #
  def first=(str, separator=$/)
    separator = /\n+/ if separator == ''
    separator = /\s+/ unless separator
    separator = Regexp.new( Regexp.escape( separator ) ) if String === separator
    i = self.index(separator)
    return nil unless i
    self[0...i] = str
  end

  # Replaces the last separation with a new string.
  # If a zero-length record separator is supplied,
  # the string is split on /\n+/. If the record 
  # separator is set to <tt>nil</tt>, then the 
  # string is split on characters.  #
  #
  #   require 'facet/string/first&last'
  #
  #   $/ = ''
  #   "Hello World".last = ", Bye."  #=>  "Hello World, Bye."
  #
  def last=(str, separator=$/)
    separator = /\n+/ if separator == ''
    separator = /\s+/ unless separator
    separator = Regexp.new( Regexp.escape( separator ) ) if String === separator
    i = self.rindex(separator)
    return nil unless i
    self[$~.end(0)..-1] = str
  end

  # Prepends a string to front of the string.
  #
  #   require 'facet/string/first&last'
  #
  #   "Hello World".first = "Hello,"  #=>  "Hello, Hello World"
  #
  def first!(str)
    self.replace(str + self)
  end
    
  # Appends a string to end of the string.
  #
  #   require 'facet/string/first&last'
  #
  #   "Hello World".last = ". Bye."  #=>  "Hello World. Bye."
  #
  def last!(str)
    self << str
  end
  
  # Returns first _n_ characters.
  #
  #   require 'facet/string/first&last'
  #
  #   "Hello World".first(3)  #=> "Hel"
  #
  def first_char(n=nil)
    n = n.to_i
    return self.slice(0,n)
  end

  # Returns last _n_ characters.
  #
  #   require 'facet/string/first&last'
  #
  #   "Hello World".last(3)  #=> "rld"
  #
  def last_char(n=nil)
    n = n.to_i
    return self[-n..-1]
  end

end
